@@ -23,7 +23,6 @@ G_BEGIN_DECLS
 
 #include <gst/gst.h>
 #include <gst/video/video.h>
-#include <gst/video/gstvideodecoder.h>
 #include <libavcodec/avcodec.h>
 
 typedef struct _GstFFMpegVidDec GstFFMpegVidDec;
@@ -61,7 +60,7 @@ struct _GstFFMpegVidDec
   GstBuffer *palette;
 
   guint8 *padded;
-  guint padded_size;
+  gint padded_size;
 
   /* some properties */
   enum AVDiscard skip_frame;
